@@ -11,19 +11,22 @@ from smarkets.streaming_api import seto
 from smarkets.streaming_api.exceptions import InvalidCallbackError
 
 
-_ETO_PAYLOAD_TYPES = dict((
-    (getattr(eto, x), 'eto.%s' % x.replace('PAYLOAD_', '').lower())
-    for x in dir(eto) if x.startswith('PAYLOAD_')
-))
+def _get_payload_types(module):
+    module_name = module.__name__.split('.')[-1]
+
+    return dict((
+        (getattr(module, x),
+         '%s.%s' % (module_name, x.replace('PAYLOAD_', '').lower()))
+        for x in dir(module) if x.startswith('PAYLOAD_')
+    ))
 
 
-_SETO_PAYLOAD_TYPES = dict((
-    (getattr(seto, x), 'seto.%s' % x.replace('PAYLOAD_', '').lower())
-    for x in dir(seto) if x.startswith('PAYLOAD_')
-))
+_ETO_PAYLOAD_TYPES = _get_payload_types(eto)
+_SETO_PAYLOAD_TYPES = _get_payload_types(seto)
 
 
 class StreamingAPIClient(object):
+
     """
     Smarkets API implementation
 
@@ -33,10 +36,10 @@ class StreamingAPIClient(object):
 
     logger = logging.getLogger(__name__ + '.SETOClient')
 
-    def __init__(self, session, auto_flush=True):
+    def __init__(self, session):
         self.session = session
-        self.auto_flush = auto_flush
-        self.callbacks = dict((callback_name, Signal()) for callback_name in self.__class__.CALLBACKS)
+        self.callbacks = dict((callback_name, Signal())
+                              for callback_name in self.__class__.CALLBACKS)
         self.global_callback = Signal()
 
     def login(self, receive=True):
@@ -44,6 +47,7 @@ class StreamingAPIClient(object):
         self.session.connect()
         if receive:
             self.read()
+            self.flush()
 
     def logout(self, receive=True):
         """
@@ -53,14 +57,43 @@ class StreamingAPIClient(object):
         self.session.logout()
         if receive:
             self.read()
+            self.flush()
+
         self.session.disconnect()
 
-    def read(self, num=1):
-        "Receive the next `num` payloads and block"
-        for _ in xrange(0, num):
+    @property
+    def raw_socket(self):
+        """
+        Get raw socket used for communication with remote endpoint.
+
+        :rtype: :class:`socket.socket`
+        """
+        return self.session.raw_socket
+
+    @property
+    def output_buffer_size(self):
+        return self.session.output_buffer_size
+
+    def read(self):
+        """
+        .. note::
+            This method will block until it can read *any* data from the remote endpoint. It doesn't mean
+            it will receive enough data to process it.
+        :return: Number of processed incoming messages.
+        :rtype: int
+        """
+        self.session.read()
+
+        processed = 0
+        while True:
             frame = self.session.next_frame()
             if frame:
                 self._dispatch(frame)
+                processed += 1
+            else:
+                break
+
+        return processed
 
     def flush(self):
         "Flush the send buffer"
@@ -70,10 +103,7 @@ class StreamingAPIClient(object):
         payload = self.session.out_payload
         payload.Clear()
         message.copy_to(payload)
-        seq = self._send()
-        if hasattr(message, 'register_callbacks'):
-            message.register_callbacks(self)
-        return seq
+        self._send()
 
     def ping(self):
         "Ping the service"
@@ -81,7 +111,7 @@ class StreamingAPIClient(object):
         msg.Clear()
         msg.type = seto.PAYLOAD_ETO
         msg.eto_payload.type = eto.PAYLOAD_PING
-        return self._send()
+        self._send()
 
     def subscribe(self, market):
         "Subscribe to a market"
@@ -89,21 +119,21 @@ class StreamingAPIClient(object):
         msg.Clear()
         msg.type = seto.PAYLOAD_MARKET_SUBSCRIBE
         msg.market_subscribe.market.CopyFrom(market)
-        return self._send()
+        self._send()
 
     def request_account_state(self):
         "Request Account State"
         msg = self.session.out_payload
         msg.Clear()
         msg.type = seto.PAYLOAD_ACCOUNT_STATE_REQUEST
-        return self._send()
+        self._send()
 
     def request_orders_for_account(self):
         "Request an account's orders"
         msg = self.session.out_payload
         msg.Clear()
         msg.type = seto.PAYLOAD_ORDERS_FOR_ACCOUNT_REQUEST
-        return self._send()
+        self._send()
 
     def request_orders_for_market(self, market):
         "Request an account's orders for a market"
@@ -111,7 +141,7 @@ class StreamingAPIClient(object):
         msg.Clear()
         msg.type = seto.PAYLOAD_ORDERS_FOR_MARKET_REQUEST
         msg.orders_for_market_request.market.CopyFrom(market)
-        return self._send()
+        self._send()
 
     def unsubscribe(self, market):
         "Unsubscribe from a market"
@@ -119,14 +149,14 @@ class StreamingAPIClient(object):
         msg.Clear()
         msg.type = seto.PAYLOAD_MARKET_UNSUBSCRIBE
         msg.market_unsubscribe.market.CopyFrom(market)
-        return self._send()
+        self._send()
 
     def request_events(self, request):
         "Send a structured events request"
         msg = self.session.out_payload
         msg.Clear()
         request.copy_to(msg)
-        return self._send()
+        self._send()
 
     def add_handler(self, name, callback):
         "Add a callback handler"
@@ -178,10 +208,9 @@ class StreamingAPIClient(object):
 
     def _send(self):
         """
-        Send a payload via the session and return the sequence number
-        used for the outgoing payload
+        Send a payload via the session.
         """
-        return self.session.send(self.auto_flush)
+        self.session.send()
 
     def _dispatch(self, message):
         "Dispatch a frame to the callbacks"
